@@ -1,12 +1,12 @@
 from .integration import SessionDep, attach_db_to_api, attach_db_to_api_with_url
 from .health import db_health_router
-from .crud import Resources, include_crud
+from .include import Resource, include_resources
 
 __all__ = [
     "SessionDep",
     "attach_db_to_api",
     "attach_db_to_api_with_url",
     "db_health_router",
-    "Resources",
-    "include_crud",
-]
+    "Resource",
+    "include_resources",
+]
\ No newline at end of file
